@@ -33,7 +33,11 @@ idaNaiveBayes <- function(form, data, id="id", modelname=NULL) {
     }
     
     xx <- parseTableName(modelname);
-    model <- paste('"',xx$schema,'"."',xx$table,'"',sep=''); 	
+    if (idaIsDb2z()) {
+		model <- paste('"',xx$table,'"',sep=''); 	
+	} else {
+		model <- paste('"',xx$schema,'"."',xx$table,'"',sep=''); 	
+	}
   }
   
   
@@ -47,7 +51,7 @@ idaNaiveBayes <- function(form, data, id="id", modelname=NULL) {
   id  <- paste('"',id,'"',sep='');
   
   tryCatch({
-        callSP("IDAX.NAIVEBAYES ", model=model, intable=tmpView, id=id, target=varY)
+        callSP("NAIVEBAYES", model=model, intable=tmpView, id=id, target=varY)
       }, error = function(e, tmpView) {
         # in case of error, drop view and let user know, what happend
         stop(e)
@@ -70,11 +74,39 @@ idaRetrieveNBModel <- function(modelName) {
   model <- xx$table
   modelSchema <- xx$schema
   
-  
   # results are converted to naiveBayes object
-  model2 <- paste('SELECT * FROM "',modelSchema,'"."',model,'_MODEL"',sep="")
-  nbOut <- idaQuery(model2);
-  
+  modelColList <- "ATTRIBUTE,  VAL,  CLASS, CLASSVALCOUNT, ATTRCLASSCOUNT, CLASSCOUNT, TOTALCOUNT "
+  discColList <- "COLNAME, BREAK "
+  splits.rcv <- NULL
+  if(idaIsDb2z()) {     
+	exportModelTable <- idaGetValidTableName(prefix = "IDAR_MODEL_TABLE_")
+	tryCatch({	
+        res <- callSP("EXPORT_MODEL_TO_TABLE",
+            			model=modelName,
+            			outtable=exportModelTable)
+		model2 <- paste('SELECT ', modelColList, ' FROM ', exportModelTable,' where MODELUSAGE= \'Model\'',sep="")
+		nbOut <- idaQuery(model2)
+		# read the DISC model columns if they exist 
+		if (idaExistColumnInTable("BREAK", exportModelTable)) {
+			splits.rcv <- idaQuery(paste('SELECT ', discColList, ' FROM ', exportModelTable,' where MODELUSAGE= \'DiscRanges\'',sep=""))
+		}	
+      }, error = function(e) {
+        # in case of error, let user know what happend
+        stop(e)
+      }, finally = {
+        idaDeleteTable(exportModelTable)
+      }
+	) 
+  } else { 
+    model2 <- paste('SELECT * FROM "',modelSchema,'"."',model,'_MODEL"',sep="")
+    nbOut <- idaQuery(model2); 
+    # read the DISC model table if it exists 
+    disc.name <- paste('"',modelSchema,'"."',model,'_DISC"',sep="")
+    if (idaExistTable(disc.name)) {
+		splits.rcv <- idaQuery(paste('SELECT * FROM "',modelSchema,'"."',model,'_DISC"',sep=""))
+	}
+  }
+	
   nbOut$CLASSVALCOUNT <- as.numeric(nbOut$CLASSVALCOUNT)
   nbOut$ATTRCLASSCOUNT <- as.numeric(nbOut$ATTRCLASSCOUNT)
   nbOut$CLASSCOUNT <- as.numeric(nbOut$CLASSCOUNT)
@@ -84,7 +116,8 @@ idaRetrieveNBModel <- function(modelName) {
   val <- "VAL"
   attribute <- "ATTRIBUTE"
   classvalcount <- "CLASSVALCOUNT"
-  attrclasscount <- "ATTRCLASSCOUNT"
+  attrclasscount <- "ATTRCLASSCOUNT"	
+   
   colname <- "COLNAME"
   totalcount <- "TOTALCOUNT"
   
@@ -101,20 +134,20 @@ idaRetrieveNBModel <- function(modelName) {
     names.list[[xVar]] <- values
     dimnames(tab) <- names.list
     for (j in 1:nrow(tmp)) {
-      tab[tmp[j,class], tmp[j,val]] <- as.numeric(tmp[j,classvalcount])/tmp[j,"CLASSCOUNT"]
+      #  tab[tmp[j,class], tmp[j,val]] <- as.numeric(tmp[j,classvalcount])/tmp[j,"CLASSCOUNT"]
+      tab[tmp[j,class], tmp[j,val]] <- as.numeric(tmp[j,classvalcount])/tmp[j, attrclasscount]
     }
     tables[[i]] <- tab
     names(tables)[i] <- xVar
   }
-  splits.rcv <- NULL
+ 
   # check if there are automatically made splits for continuous values
-  disc.name <- paste('"',modelSchema,'"."',model,'_DISC"',sep="")
-  if (idaExistTable(disc.name)) {
-    splits.rcv <- idaQuery(paste('SELECT * FROM "',modelSchema,'"."',model,'_DISC"',sep=""))
-    classes <- unique(splits.rcv[[colname]])
+  # tb: replaced "classes" by "cols"
+  if (!is.null(splits.rcv)) {
+    cols <- unique(splits.rcv[[colname]])
     splits <- list()
-    for (i in 1:length(classes)) {
-      splits[[classes[i]]] <- splits.rcv[splits.rcv[[colname]] == classes[i], "BREAK"]
+    for (i in 1:length(cols)) {
+      splits[[cols[i]]] <- splits.rcv[splits.rcv[[colname]] == cols[i], "BREAK"]
     }
   } else {
     splits <- list()
@@ -146,11 +179,16 @@ print.idaNaiveBayes <- function (x, ...) {
 }
 
 
-predict.idaNaiveBayes <- function(object, newdata, id,...) {
+predict.idaNaiveBayes <- function(object, newdata, id, withProbabilities=FALSE, ...) {
   
   newData <- newdata
   
-  outtable <- idaGetValidTableName(paste("PREDICT_",sep=""))
+  args = list(...)
+  outtable <- args[["outtable"]]
+  outtableprob <-args[["outtableprob"]]
+  if (is.null(outtable) || outtable == "") {
+  	outtable <- idaGetValidTableName(paste("PREDICT_",sep=""))
+  }
   
   colu = newData@cols
   if (!(id %in% colu))
@@ -159,23 +197,73 @@ predict.idaNaiveBayes <- function(object, newdata, id,...) {
   id  <- paste('\"',id,'\"',sep="")
   tmpView <- idaCreateView(newData)
   
-  tryCatch({	
-          callSP("IDAX.PREDICT_NAIVEBAYES ",
+  tryCatch({
+  	if (withProbabilities) {
+  		if (is.null(outtableprob) || outtableprob == "") {
+  			probTable <- idaGetValidTableName(paste("PROBTABLE_",sep=""))		
+  		} else {
+  			probTable <- outtableprob
+  		}
+  				
+  		outtable2 <- idaGetValidTableName(paste("PREDICT_",sep=""))			
+        callSP("PREDICT_NAIVEBAYES",
+              model=object$model,
+              intable=tmpView,
+              id=id,
+              outtableprob=probTable,
+              outtable=outtable2,
+              ...);
+ 	} else {
+ 		 callSP("PREDICT_NAIVEBAYES",
               model=object$model,
               intable=tmpView,
               id=id,
               outtable=outtable,
-              ...);
-          
-      }, error = function(e) {
+              ...); 
+    }                
+   }, error = function(e) {
         # in case of error, let user know what happend
         stop(e)
-      }, finally = {
+   }, finally = {
         # drop view
         idaDropView(tmpView)
-      }
+   }
   )
   
+  if (withProbabilities) {
+ 	tryCatch({
+  		outview <- idaGetValidTableName(paste("PREDICT_VIEW_",sep=""))
+  		createViewStmt <- paste("CREATE VIEW ", outview, " AS WITH AGGPROB(", id, ", SUMPROB) AS (SELECT ", id, ", SUM(PROB) AS SUMPROB FROM ", 
+  								probTable, " GROUP BY ", id, ")", sep = "")
+  		selectClause <- paste(" SELECT o.", id, ", o.CLASS", sep = "")
+  		fromClause <- paste(" FROM ", outtable2, " o, AGGPROB a", sep="")
+  		whereClause1 <- paste(" WHERE o.", id, "=a.", id, " AND a.", id, "=", sep="") 						 
+		whereClause2 <- ""
+  		for(i in 1:length(object$level)) {
+  			classValue <- object$level[i]
+  			selectClause <- paste(selectClause, ", pb", i, ".PROB/SUMPROB  AS \"", classValue, "\"", sep="")
+  			fromClause <- paste(fromClause, ", ", probTable, " pb", i, sep="")
+  			whereClause1 <- paste(whereClause1, "pb", i, ".", id, sep="")
+  			if(i<length(object$level)) {
+  				whereClause1 <- paste(whereClause1, " AND pb", i, ".", id, "=", sep="")
+  			} 			
+  			whereClause2 <- paste(whereClause2, " AND pb", i, ".CLASS='", classValue, "'", sep="")  
+  		}
+  		createViewStmt <- paste(createViewStmt, selectClause, fromClause, whereClause1, whereClause2, sep="")
+  		idaQuery(createViewStmt)
+  		idaMaterialize(ida.data.frame(outview), outtable, asAOT=TRUE)	
+   	}, error = function(e) {
+        # in case of error, let user know what happend
+        stop(e)
+   	}, finally = {
+        # drop view
+        idaDropView(outview)
+        idaDeleteTable(outtable2)
+        if (is.null(outtableprob) || outtableprob == "") {
+			idaDeleteTable(probTable)
+        }
+   	})
+  }   	 
   object.pred <- ida.data.frame(outtable)
   return(object.pred)
 }
